#!/usr/bin/perl
# Author: Mike Brodbelt
# Date: 12/08/2011

use strict;

#use Data::Dumper;

# Small script to read varnish statistics from a varnish cache. We
# require a working local varnishadm, with appropriate authentication
# credentials set up. We implement multiple "stats_set" options, each
# of which returns a small subset of varnishstats suitable for use in
# a Cacti data template.

# Statistics sets - we define a hash mapping shortnames for cacti output
# to varnish statistic names here, for each statistic set we want this
# script to return.

my %backends = ( "bck_conn_succ" => "Backend conn. success",
		 "bck_conn_fail" => "Backend conn. failures",
		 "bck_config"    => "N backends",
		 "bck_reqs"      => "Backend requests made" );

my %conns = ( "conns_accept"	=> "Client connections accepted",
	      "conns_dropped"	=> "Connection dropped, no sess/wrk",
	      "client_drops"	=> "Connection dropped late" );


my %threads = ( "thrd_cnt"      => "N worker threads",
		"thrds_create"  => "N worker threads created",
		"thrds_limited" => "N worker threads limited" );

my %workqueue = ( "queued_reqs"     => "N queued work requests",
		  "overflowed_reqs" => "N overflowed work requests",
		  "dropped_reqs"    => "N dropped work requests" );

my %cachebytes = ( "hdr_bytes"  => "Total header bytes",
		   "body_bytes" => "Total body bytes" );

my %cachereqs = ( "cache_hits"     => "Cache hits",
		  "cache_neg_hits" => "Cache hits for pass",
		  "cache_misses"   => "Cache misses" );

my %cachesize = ( "cache_bytes" => "bytes allocated",
		  "cache_free"  => "bytes free" );

my %cacheobjs = ( "objs_expired" => "N expired objects",
		  "objs_nuked"   => "N LRU nuked objects",
		  "objs_moved"   => "N LRU moved objects" );


########################################################################

my $usage = "\nusage: $0 <host> <port> <key> <stats_set>\n\n";

@ARGV == 4 or die "$usage";

my $varnishadm = "/usr/bin/varnishadm";
my $keydir = "/var/www/cacti/auth/varnish/";
my $vadmpid;
my %vstats = ();
my @cacti_out;
my ($host, $port, $keyfile, $stats)  = @ARGV;

if ($vadmpid = open(VARNISHADM, "-|")) {
	while (<VARNISHADM>) {
		my @fields;
		chomp; s/^\s+//; s/\s+$//;
		@fields = split (/\W*\s+\W*/, $_, 2);
		$vstats{$fields[1]} = $fields[0];
	}
	close(VARNISHADM);
} else {
	die "can't fork: $!" unless defined $vadmpid;
	exec( "$varnishadm", "-T", "${host}:${port}", "-S", "${keydir}${keyfile}.key", "stats" );
}

if ($stats eq "backends") {
	foreach my $val (keys %backends) {
		push @cacti_out, "$val:$vstats{ $backends{$val} }";
	}
} elsif ($stats eq "conns") {
	foreach my $val (keys %conns) {
		push @cacti_out, "$val:$vstats{ $conns{$val} }";
	}
} elsif ($stats eq "threads") {
	foreach my $val (keys %threads) {
		push @cacti_out, "$val:$vstats{ $threads{$val} }";
	}
} elsif ($stats eq "workqueue") {
	foreach my $val (keys %workqueue) {
		push @cacti_out, "$val:$vstats{ $workqueue{$val} }";
	}
} elsif ($stats eq "cachebytes") {
	foreach my $val (keys %cachebytes) {
		push @cacti_out, "$val:$vstats{ $cachebytes{$val} }";
	}
} elsif ($stats eq "cachereqs") {
	foreach my $val (keys %cachereqs) {
		push @cacti_out, "$val:$vstats{ $cachereqs{$val} }";
	}
} elsif ($stats eq "cachesize") {
	foreach my $val (keys %cachesize) {
		push @cacti_out, "$val:$vstats{ $cachesize{$val} }";
	}
} elsif ($stats eq "cacheobjs") {
	foreach my $val (keys %cacheobjs) {
		push @cacti_out, "$val:$vstats{ $cacheobjs{$val} }";
	}
}

print join(" ", @cacti_out) . "\n";
